@@ -1,8 +1,10 @@
 import xml.etree.ElementTree as ElementTree
 from xml.dom import minidom
 
+from typecraft_python.globals import STRICT_MODE
 from typecraft_python.exceptions.parsing import TypecraftParseException
 from typecraft_python.models import Text, Phrase, Word, Morpheme, GlobalTagSet, GlobalTag
+from typecraft_python.models import Text, Phrase, Word, Morpheme, GlobalTagSet, GlobalTag, PhraseValidity
 from typecraft_python.globals import *
 
 """
@@ -44,7 +46,6 @@ class _ParserHelper:
         """
         pass
 
-
     @staticmethod
     def check_text_for_conformity(text_root):
         """
@@ -181,7 +182,7 @@ class _ParserHelper:
         lang = text_root.attrib.get('lang')
 
         if body is not None:
-            text.body = body
+            text.rich_text = body.text
 
         if id is not None:
             text.id = id
@@ -233,23 +234,36 @@ class _ParserHelper:
 
         id = phrase_root.attrib.get('id')
         validity = phrase_root.attrib.get('valid')
+        description = phrase_root.find(ns + 'description')
         translation_tree = phrase_root.find(ns + 'translation')
+        translation2_tree = phrase_root.find(ns + 'translation2')
         globaltags_tree = phrase_root.find(ns + 'globaltags')
 
         if id is not None:
             phrase.id = id
 
         if validity is not None:
-            phrase.validity = validity
+            if isinstance(validity, int):
+                phrase.validity = validity
+            elif hasattr(PhraseValidity, validity):
+                phrase.validity = getattr(PhraseValidity, validity)
+            else:
+                phrase.validity = PhraseValidity.UNKNOWN
 
         if translation_tree is not None:
             phrase.translation = translation_tree.text if translation_tree.text is not None else ""
 
+        if translation2_tree is not None:
+            phrase.translation2 = translation2_tree.text if translation2_tree.text is not None else ""
+
         if globaltags_tree is not None:
             phrase.global_tag_set = GlobalTagSet(globaltags_tree.attrib.get('id'), globaltags_tree.attrib.get('tagset'))
 
-        for global_tag in globaltags_tree.findall(ns + 'globaltag'):
-            phrase.add_global_tag(GlobalTag(name=global_tag.text, level=global_tag.attrib.get('level')))
+            for global_tag in globaltags_tree.findall(ns + 'globaltag'):
+                phrase.add_global_tag(GlobalTag(name=global_tag.text, level=global_tag.attrib.get('level')))
+
+        if description is not None:
+            phrase.comment = description.text if description is not None else ""
 
         return
 
@@ -511,9 +525,20 @@ class Parser:
         ElementTree.SubElement(text_el, 'titleTranslation').text = text.title_translation
         ElementTree.SubElement(text_el, 'body').text = text.rich_text
 
+        Parser.convert_text_metadata_to_etree(text_el, text)
+
         for phrase in text:
             Parser.convert_phrase_to_etree(text_el, phrase)
 
+    @staticmethod
+    def convert_text_metadata_to_etree(root, text):
+        if not text.metadata or not isinstance(text.metadata, dict) or len(text.metadata) == 0:
+            return
+
+        metadata_el = ElementTree.SubElement(root, 'extraMetadata')
+        for key, val in text.metadata.items():
+            ElementTree.SubElement(metadata_el, 'metadata', {'name': key}).text = val
+
     @staticmethod
     def convert_phrase_to_etree(root, phrase):
         assert isinstance(phrase, Phrase)
@@ -521,11 +546,11 @@ class Parser:
         phrase_el = ElementTree.SubElement(root, 'phrase', {'valid': 'UNKNOWN'})
 
         ElementTree.SubElement(phrase_el, 'original').text = phrase.phrase
-        ElementTree.SubElement(phrase_el, 'translation').text = phrase.free_translation
-        ElementTree.SubElement(phrase_el, 'translation2').text = phrase.free_translation2
+        ElementTree.SubElement(phrase_el, 'translation').text = phrase.translation
+        ElementTree.SubElement(phrase_el, 'translation2').text = phrase.translation2
         global_tags_el = ElementTree.SubElement(phrase_el, 'globaltags', {
             'id': str(phrase.global_tag_set.id), 'tagset': phrase.global_tag_set.name})
-        ElementTree.SubElement(phrase_el, 'comment').text = phrase.comment
+        ElementTree.SubElement(phrase_el, 'description').text = phrase.comment
 
         for word in phrase:
             Parser.convert_word_to_etree(phrase_el, word)
